using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI;

namespace ReinforcementCall
{
	public class Building_RedPhoneRC : Building
	{
		private CompPowerTrader powerComp;

		public bool CanUseCommsNow
		{
			get
			{
				if (base.Spawned && base.Map.gameConditionManager.ConditionIsActive(GameConditionDefOf.SolarFlare))
				{
					return false;
				}
				return powerComp.PowerOn;
			}
		}

		public override void SpawnSetup(Map map, bool respawningAfterLoad)
		{
			base.SpawnSetup(map, respawningAfterLoad);
			powerComp = GetComp<CompPowerTrader>();
			LessonAutoActivator.TeachOpportunity(ConceptDefOf.OpeningComms, OpportunityType.GoodToKnow);
		}

		private FloatMenuOption GetFailureReason(Pawn myPawn)
		{
			if (!ReachabilityUtility.CanReach(myPawn, (LocalTargetInfo)this, PathEndMode.InteractionCell, Danger.Some, false))
			{
				return new FloatMenuOption("CannotUseNoPath".Translate(), (Action)null, MenuOptionPriority.Default);
			}
			if (base.Spawned && base.Map.gameConditionManager.ConditionIsActive(GameConditionDefOf.SolarFlare))
			{
				return new FloatMenuOption("CannotUseSolarFlare".Translate(), (Action)null, MenuOptionPriority.Default);
			}
			if (!powerComp.PowerOn)
			{
				return new FloatMenuOption((string)"CannotUseNoPower".Translate(), (Action)null, MenuOptionPriority.Default);
			}
			if (!myPawn.health.capacities.CapableOf(PawnCapacityDefOf.Talking))
			{
				return new FloatMenuOption((string)"CannotUseReason".Translate("IncapableOfCapacity".Translate(PawnCapacityDefOf.Talking.label)), (Action)null, MenuOptionPriority.Default);
			}
			if (myPawn.skills.GetSkill(SkillDefOf.Social).TotallyDisabled)
			{
				return new FloatMenuOption((string)"CannotPrioritizeWorkTypeDisabled".Translate(SkillDefOf.Social.LabelCap), (Action)null, MenuOptionPriority.Default);
			}
			if (!CanUseCommsNow)
			{
				Log.Error(myPawn?.ToString() + " could not use comm console for unknown reason.", ignoreStopLoggingLimit: false);
				return new FloatMenuOption("Cannot use now", (Action)null, MenuOptionPriority.Default);
			}
			return null;
		}

		public IEnumerable<Faction> GetCommTargets(Pawn myPawn)
		{
			IEnumerable<Faction> source = Find.FactionManager.AllFactionsVisibleInViewOrder.Where((Faction f) => !f.IsPlayer);
			return source.Where((Faction f) => (f.PlayerRelationKind == FactionRelationKind.Neutral && f.PlayerGoodwill > 0) || (f.PlayerRelationKind == FactionRelationKind.Ally && !f.IsPlayer));
		}

		public override IEnumerable<FloatMenuOption> GetFloatMenuOptions(Pawn myPawn)
		{
			FloatMenuOption failureReason = GetFailureReason(myPawn);
			if (failureReason != null)
			{
				yield return failureReason;
				yield break;
			}
			IEnumerable<Faction> factionsToCall = GetCommTargets(myPawn);
			if (factionsToCall == null)
			{
				yield return new FloatMenuOption((string)"NoFriendlyFactionRC".Translate(), (Action)null, MenuOptionPriority.Default);
			}
			else if (factionsToCall.Count() < 1)
			{
				yield return new FloatMenuOption((string)"NoFriendlyFactionRC".Translate(), (Action)null, MenuOptionPriority.Default);
			}
			foreach (Faction commTarget in factionsToCall)
			{
				FloatMenuOption option = CommFloatMenuOption(commTarget, myPawn);
				if (option != null)
				{
					yield return option;
				}
			}
		}

		public FloatMenuOption CommFloatMenuOption(Faction commTarget, Pawn negotiator)
		{
			if (commTarget.IsPlayer)
			{
				return null;
			}
			string text = "CallOnRadio".Translate(commTarget.GetCallLabel());
			string text2 = text;
			text = text2 + " (" + commTarget.PlayerRelationKind.GetLabel() + ", " + commTarget.PlayerGoodwill.ToStringWithSign() + ")";
			if (LeaderIsAvailableToTalk(commTarget))
			{
				return FloatMenuUtility.DecoratePrioritizedTask(new FloatMenuOption(text, (Action)delegate
				{
					GiveUseCommsJob(negotiator, commTarget);
				}, MenuOptionPriority.InitiateSocial), negotiator, this);
			}
			string text3 = ((commTarget.leader == null) ? ((string)"LeaderUnavailableNoLeader".Translate()) : ((string)"LeaderUnavailable".Translate(commTarget.leader.LabelShort, commTarget.leader)));
			return new FloatMenuOption(text + " (" + text3 + ")", (Action)null, MenuOptionPriority.Default);
		}

		public bool LeaderIsAvailableToTalk(Faction faction)
		{
			bool result;
			if (faction.leader == null)
			{
				result = false;
			}
			else
			{
				if (faction.leader.Spawned && (faction.leader.Downed || faction.leader.IsPrisoner || !faction.leader.Awake() || faction.leader.InMentalState))
				{
					return false;
				}
				result = true;
			}
			return result;
		}

		public void GiveUseCommsJob(Pawn negotiator, Faction target)
		{
			Job job = new Job(RCDefsOf.UseRedPhoneRC, this);
			job.commTarget = target;
			negotiator.jobs.TryTakeOrderedJob(job, JobTag.Misc);
			PlayerKnowledgeDatabase.KnowledgeDemonstrated(ConceptDefOf.OpeningComms, KnowledgeAmount.Total);
		}
	}
}
