using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;
using Verse.AI;
using Verse.Sound;

namespace ReinforcementCall
{
	public static class ReinforcementDialogMakerRC
	{
		public static DiaNode FactionDialogFor(Pawn negotiator, Faction faction)
		{
			Map map = negotiator.Map;
			Pawn pawn;
			string text;
			if (faction.leader != null)
			{
				pawn = faction.leader;
				text = faction.leader.Name.ToStringFull;
			}
			else
			{
				Log.Error("Faction " + faction?.ToString() + " has no leader.", ignoreStopLoggingLimit: false);
				pawn = negotiator;
				text = faction.Name;
			}
			DiaNode diaNode;
			if (faction.PlayerRelationKind != 0)
			{
				diaNode = ((faction.PlayerRelationKind != FactionRelationKind.Neutral) ? new DiaNode("FactionGreetingWarm".Translate(text, negotiator.LabelShort, negotiator.Named("NEGOTIATOR"), pawn.Named("LEADER")).AdjustedFor(pawn)) : new DiaNode("FactionGreetingWary".Translate(text, negotiator.LabelShort, negotiator.Named("NEGOTIATOR"), pawn.Named("LEADER")).AdjustedFor(pawn)));
			}
			else
			{
				string key = ((faction.def.permanentEnemy || !"FactionGreetingHostileAppreciative".CanTranslate()) ? "FactionGreetingHostile" : "FactionGreetingHostileAppreciative");
				diaNode = new DiaNode(key.Translate(text).AdjustedFor(pawn));
			}
			if (map?.IsPlayerHome ?? false)
			{
				diaNode.options.Add(RequestFreeReinforcements(map, faction, negotiator));
				if ((int)faction.def.techLevel >= 4)
				{
					diaNode.options.Add(RequestQuickReactionReinforcements(map, faction, negotiator));
					diaNode.options.Add(RequestStrongQuickReactionReinforcements(map, faction, negotiator));
				}
			}
			DiaOption diaOption = new DiaOption("(" + "Disconnect".Translate() + ")");
			diaOption.resolveTree = true;
			diaNode.options.Add(diaOption);
			return diaNode;
		}

		public static DiaOption RequestFreeReinforcements(Map map, Faction faction, Pawn negotiator)
		{
			string text = "RequestFreeMilitaryAidRC".Translate();
			if (faction.PlayerRelationKind != FactionRelationKind.Ally)
			{
				DiaOption diaOption = new DiaOption(text);
				diaOption.Disable("MustBeAlly".Translate());
				return diaOption;
			}
			if (!faction.def.allowedArrivalTemperatureRange.ExpandedBy(-4f).Includes(map.mapTemperature.SeasonalTemp))
			{
				DiaOption diaOption2 = new DiaOption(text);
				diaOption2.Disable("BadTemperature".Translate());
				return diaOption2;
			}
			int num = faction.lastMilitaryAidRequestTick + 30000 - Find.TickManager.TicksGame;
			if (num > 0)
			{
				DiaOption diaOption3 = new DiaOption(text);
				diaOption3.Disable("WaitTime".Translate(num.ToStringTicksToPeriod()));
				return diaOption3;
			}
			if (NeutralGroupIncidentUtility.AnyBlockingHostileLord(map, faction))
			{
				DiaOption diaOption4 = new DiaOption(text);
				diaOption4.Disable("HostileVisitorsPresent".Translate());
				return diaOption4;
			}
			DiaOption diaOption5 = new DiaOption(text);
			IEnumerable<Faction> source = (from x in map.attackTargetsCache.TargetsHostileToColony
				where GenHostility.IsActiveThreatToPlayer(x)
				select ((Thing)x).Faction into x
				where x != null && !x.HostileTo(faction)
				select x).Distinct();
			if (source.Any())
			{
				DiaNode diaNode = new DiaNode("MilitaryAidConfirmMutualEnemy".Translate(faction.Name, GenText.ToCommaList(source.Select((Faction fa) => fa.Name), true)));
				DiaOption diaOption6 = new DiaOption("CallConfirm".Translate());
				diaOption6.action = delegate
				{
					CallForSmallAid(map, faction);
				};
				diaOption6.resolveTree = true;
				DiaOption diaOption7 = new DiaOption("CallCancel".Translate());
				diaOption7.linkLateBind = ResetToRoot(faction, negotiator);
				diaNode.options.Add(diaOption6);
				diaNode.options.Add(diaOption7);
				diaOption5.link = diaNode;
			}
			else
			{
				diaOption5.action = delegate
				{
					CallForSmallAid(map, faction);
				};
				diaOption5.resolveTree = true;
			}
			return diaOption5;
		}

		public static DiaOption RequestQuickReactionReinforcements(Map map, Faction faction, Pawn negotiator)
		{
			int silverToPay = 800;
			int num = 25;
			string text = "RequestPaidMilitaryAidRC".Translate(silverToPay);
			if (faction.PlayerRelationKind != FactionRelationKind.Ally && faction.PlayerGoodwill < num)
			{
				DiaOption diaOption = new DiaOption(text);
				diaOption.Disable("NeedGoodwill".Translate(num.ToString("F0")));
				return diaOption;
			}
			if (AmountSendableSilver(map) < silverToPay)
			{
				DiaOption diaOption2 = new DiaOption(text);
				diaOption2.Disable("NeedSilverLaunchable".Translate(silverToPay.ToString()));
				return diaOption2;
			}
			if (!faction.def.allowedArrivalTemperatureRange.ExpandedBy(-4f).Includes(map.mapTemperature.SeasonalTemp))
			{
				DiaOption diaOption3 = new DiaOption(text);
				diaOption3.Disable("BadTemperature".Translate());
				return diaOption3;
			}
			int num2 = faction.lastMilitaryAidRequestTick + 60000 - Find.TickManager.TicksGame;
			if (num2 > 0)
			{
				DiaOption diaOption4 = new DiaOption(text);
				diaOption4.Disable("WaitTime".Translate(num2.ToStringTicksToPeriod()));
				return diaOption4;
			}
			if (NeutralGroupIncidentUtility.AnyBlockingHostileLord(map, faction))
			{
				DiaOption diaOption5 = new DiaOption(text);
				diaOption5.Disable("HostileVisitorsPresent".Translate());
				return diaOption5;
			}
			DiaOption diaOption6 = new DiaOption(text);
			IEnumerable<Faction> source = (from x in map.attackTargetsCache.TargetsHostileToColony
				where GenHostility.IsActiveThreatToPlayer(x)
				select ((Thing)x).Faction into x
				where x != null && !x.HostileTo(faction)
				select x).Distinct();
			if (source.Any())
			{
				DiaNode diaNode = new DiaNode("MilitaryAidConfirmMutualEnemy".Translate(faction.Name, GenText.ToCommaList(source.Select((Faction fa) => fa.Name), true)));
				DiaOption diaOption7 = new DiaOption("CallConfirm".Translate());
				diaOption7.action = delegate
				{
					CallForMediumAid(map, faction);
				};
				diaOption7.resolveTree = true;
				DiaOption diaOption8 = new DiaOption("CallCancel".Translate());
				diaOption8.linkLateBind = ResetToRoot(faction, negotiator);
				diaNode.options.Add(diaOption7);
				diaNode.options.Add(diaOption8);
				diaOption6.link = diaNode;
			}
			else
			{
				diaOption6.action = delegate
				{
					TradeUtility.LaunchThingsOfType(ThingDefOf.Silver, silverToPay, map, null);
					CallForMediumAid(map, faction);
				};
				diaOption6.resolveTree = true;
			}
			return diaOption6;
		}

		public static DiaOption RequestStrongQuickReactionReinforcements(Map map, Faction faction, Pawn negotiator)
		{
			int silverToPay = 1500;
			int num = 25;
			string text = "RequestHeavyPaidMilitaryAidRC".Translate(silverToPay);
			if (faction.PlayerRelationKind != FactionRelationKind.Ally && faction.PlayerGoodwill < num)
			{
				DiaOption diaOption = new DiaOption(text);
				diaOption.Disable("NeedGoodwill".Translate(num.ToString("F0")));
				return diaOption;
			}
			if (AmountSendableSilver(map) < silverToPay)
			{
				DiaOption diaOption2 = new DiaOption(text);
				diaOption2.Disable("NeedSilverLaunchable".Translate(silverToPay.ToString()));
				return diaOption2;
			}
			if (!faction.def.allowedArrivalTemperatureRange.ExpandedBy(-4f).Includes(map.mapTemperature.SeasonalTemp))
			{
				DiaOption diaOption3 = new DiaOption(text);
				diaOption3.Disable("BadTemperature".Translate());
				return diaOption3;
			}
			int num2 = faction.lastMilitaryAidRequestTick + 90000 - Find.TickManager.TicksGame;
			if (num2 > 0)
			{
				DiaOption diaOption4 = new DiaOption(text);
				diaOption4.Disable("WaitTime".Translate(num2.ToStringTicksToPeriod()));
				return diaOption4;
			}
			if (NeutralGroupIncidentUtility.AnyBlockingHostileLord(map, faction))
			{
				DiaOption diaOption5 = new DiaOption(text);
				diaOption5.Disable("HostileVisitorsPresent".Translate());
				return diaOption5;
			}
			DiaOption diaOption6 = new DiaOption(text);
			IEnumerable<Faction> source = (from x in map.attackTargetsCache.TargetsHostileToColony
				where GenHostility.IsActiveThreatToPlayer(x)
				select ((Thing)x).Faction into x
				where x != null && !x.HostileTo(faction)
				select x).Distinct();
			if (source.Any())
			{
				DiaNode diaNode = new DiaNode("MilitaryAidConfirmMutualEnemy".Translate(faction.Name, GenText.ToCommaList(source.Select((Faction fa) => fa.Name), true)));
				DiaOption diaOption7 = new DiaOption("CallConfirm".Translate());
				diaOption7.action = delegate
				{
					CallForStrongAid(map, faction);
				};
				diaOption7.resolveTree = true;
				DiaOption diaOption8 = new DiaOption("CallCancel".Translate());
				diaOption8.linkLateBind = ResetToRoot(faction, negotiator);
				diaNode.options.Add(diaOption7);
				diaNode.options.Add(diaOption8);
				diaOption6.link = diaNode;
			}
			else
			{
				diaOption6.action = delegate
				{
					TradeUtility.LaunchThingsOfType(ThingDefOf.Silver, silverToPay, map, null);
					CallForStrongAid(map, faction);
				};
				diaOption6.resolveTree = true;
			}
			return diaOption6;
		}

		private static int AmountSendableSilver(Map map)
		{
			return (from t in TradeUtility.AllLaunchableThingsForTrade(map)
				where t.def == ThingDefOf.Silver
				select t).Sum((Thing t) => t.stackCount);
		}

		private static void CallForSmallAid(Map map, Faction faction)
		{
			IncidentParms incidentParms = new IncidentParms();
			incidentParms.target = map;
			incidentParms.faction = faction;
			incidentParms.points = Rand.RangeInclusive(1000, 1500);
			incidentParms.raidArrivalMode = PawnsArrivalModeDefOf.EdgeWalkIn;
			faction.lastMilitaryAidRequestTick = Find.TickManager.TicksGame;
			IncidentDefOf.RaidFriendly.Worker.TryExecute(incidentParms);
			RCDefsOf.FreeReinforcementsOneShotRC.PlayOneShotOnCamera();
		}

		private static void CallForMediumAid(Map map, Faction faction)
		{
			IncidentParms incidentParms = new IncidentParms();
			incidentParms.target = map;
			incidentParms.faction = faction;
			incidentParms.points = Rand.RangeInclusive(2000, 3500);
			incidentParms.raidArrivalMode = PawnsArrivalModeDefOf.EdgeDrop;
			faction.lastMilitaryAidRequestTick = Find.TickManager.TicksGame;
			IncidentDefOf.RaidFriendly.Worker.TryExecute(incidentParms);
			RCDefsOf.MediumReinforcementsOneShotRC.PlayOneShotOnCamera();
		}

		private static void CallForStrongAid(Map map, Faction faction)
		{
			IncidentParms incidentParms = new IncidentParms();
			incidentParms.target = map;
			incidentParms.faction = faction;
			incidentParms.points = Rand.RangeInclusive(4000, 5000);
			incidentParms.raidArrivalMode = PawnsArrivalModeDefOf.EdgeDrop;
			faction.lastMilitaryAidRequestTick = Find.TickManager.TicksGame;
			IncidentDefOf.RaidFriendly.Worker.TryExecute(incidentParms);
			RCDefsOf.StrongReinforcementsOneShotRC.PlayOneShotOnCamera();
		}

		private static Func<DiaNode> ResetToRoot(Faction faction, Pawn negotiator)
		{
			return () => FactionDialogFor(negotiator, faction);
		}
	}
}
